

#include "discoord.h"


/* **********************************************************
********************************************************** */
DISPDU_entity_state_type::DISPDU_entity_state_type() {

   header.type = DIS_PDU_TYPE_ENTITY_STATE;
   header.family = DIS_PDU_FAMILY_ENTITY_INFO_INTERACT;
   header.size = DIS_PDU_SIZE_BASE_ENTITY_STATE;
   site_id = dis_site_id;
   application_id = dis_application_id;
}


/* **********************************************************
   creates and initalizes entity_state_pdu
********************************************************** */
void DISPDU_entity_state_type::setup(int id, int fid,
   DIS_entity_type_type *type, int dr_algo) {

//   header.size += asdf - no articulated parts for now
   entity_id = id;
   force_id  = fid;
   articulation_parameter_count = 0; // asdf no articulated parts for now
   entity_type = alt_entity_type = *type;
   dr_algorithm = dr_algo;
   character_set = 0;  // unused marking field
   capability_flag = 0; // no logistics capabilities
   appearance_flag = 0; // no special appearance features
}


/* **************************************************************
************************************************************** */
void DISPDU_entity_state_type::build(buffer_type *buffer) {

   sfile sbuffer;
   int i;
   
   sbuffer.dest();

   header.build(&sbuffer);

   sbuffer.sput_ushort(site_id);
   sbuffer.sput_ushort(application_id);
   sbuffer.sput_ushort(entity_id);

   sbuffer.sput_char(force_id);
   sbuffer.sput_char(articulation_parameter_count);

   entity_type.build(&sbuffer);
   alt_entity_type.build(&sbuffer);

   sbuffer.sput_float(velocity[0]);
   sbuffer.sput_float(velocity[1]);
   sbuffer.sput_float(velocity[2]);

   sbuffer.sput_double(location[0]);
   sbuffer.sput_double(location[1]);
   sbuffer.sput_double(location[2]);

   sbuffer.sput_float(ypr[0]);
   sbuffer.sput_float(ypr[1]);
   sbuffer.sput_float(ypr[2]);

   sbuffer.sput_uint(appearance_flag);

   sbuffer.sput_char(dr_algorithm);

   for (i=0; i<15; i++)
      sbuffer.sput_char(dr_other[i]);

   sbuffer.sput_float(dr_accel[0]);
   sbuffer.sput_float(dr_accel[1]);
   sbuffer.sput_float(dr_accel[2]);

   sbuffer.sput_float(dr_ypr_velocity[0]);
   sbuffer.sput_float(dr_ypr_velocity[1]);
   sbuffer.sput_float(dr_ypr_velocity[2]);

   sbuffer.sput_char(character_set);

   for (i=0; i<11; i++)
      sbuffer.sput_char(marking[i]);

   sbuffer.sput_uint(capability_flag);

   for (i=0; i<articulation_parameter_count; i++) {
//asdf put 128 bit articulation record in
   }

   buffer->buffercpy((char *)sbuffer.data, sbuffer.vsize);
}


/* **********************************************************
********************************************************** */
void DISPDU_entity_state_type::extract(buffer_type *buffer) {

   sfile sbuffer;
   int i;

   sbuffer.scan_data(buffer);

   header.extract(&sbuffer);

   site_id = sbuffer.scan_ushort();
   application_id = sbuffer.scan_ushort();
   entity_id = sbuffer.scan_ushort();

   force_id = sbuffer.scan_char();
   articulation_parameter_count = sbuffer.scan_char();

   entity_type.extract(&sbuffer);
   alt_entity_type.extract(&sbuffer);

   velocity[0] = sbuffer.scan_float();
   velocity[1] = sbuffer.scan_float();
   velocity[2] = sbuffer.scan_float();

   location[0] = sbuffer.scan_double();
   location[1] = sbuffer.scan_double();
   location[2] = sbuffer.scan_double();
   
   ypr[0] = sbuffer.scan_float();
   ypr[1] = sbuffer.scan_float();
   ypr[2] = sbuffer.scan_float();

   appearance_flag = sbuffer.scan_uint();

   dr_algorithm = sbuffer.scan_char();   

   for (i=0; i<15; i++)
      dr_other[i] = sbuffer.scan_char();

   dr_accel[0] = sbuffer.scan_float();
   dr_accel[1] = sbuffer.scan_float();
   dr_accel[2] = sbuffer.scan_float();
   
   dr_ypr_velocity[0] = sbuffer.scan_float();
   dr_ypr_velocity[1] = sbuffer.scan_float();
   dr_ypr_velocity[2] = sbuffer.scan_float();

   character_set = sbuffer.scan_char();

   for (i=0; i<11; i++)
      marking[i] = sbuffer.scan_char();

   capability_flag = sbuffer.scan_uint();

   for (i=0; i<articulation_parameter_count; i++) {
//asdf scan 128 bit articulation record in
   }

}
