

#include <string.h>

#include "discoord.h"


/* **********************************************************
   creates and initalizes fire_pdu
********************************************************** */
DISPDU_fire_type::DISPDU_fire_type() {

   header.type = DIS_PDU_TYPE_FIRE;
   header.family = DIS_PDU_FAMILY_ENTITY_INFO_INTERACT;
   header.size = DIS_PDU_SIZE_FIRE;
   fire_site_id = dis_site_id;
   fire_application_id = dis_application_id;
   ammo_site_id = dis_site_id;
   ammo_application_id = dis_application_id;
   event_site_id = dis_site_id;
   event_application_id = dis_application_id;
   fire_mission_index = FIRE_PDU_NO_FIRE_MISSION;
}
   

/* **********************************************************
   creates and initalizes fire_pdu
********************************************************** */
void DISPDU_fire_type::setup(int fire_id, DIS_id_type *target,
   int ammo_id, int event_id, DIS_burst_descriptor_type *burst,
   double *pos, double *vel, double *range, unsigned int timer) {

   header.time = timer;

   fire_entity_id = fire_id;

   target_site_id = target->site;
   target_application_id = target->application;
   target_entity_id = target->entity;
   
   if (ammo_id)
      ammo_entity_id = ammo_id;
   else {
      ammo_entity_id = 0;
      ammo_application_id = 0;
      ammo_site_id = 0;
   }

   event_entity_id = event_id;

   copyarray3(location, pos); 

   memcpy(&burst_data, burst, sizeof(DIS_burst_descriptor_type));

   copyarray3(velocity, (float)vel);

   this->range = (float)*range;
}


/* **********************************************************
********************************************************** */
void DISPDU_fire_type::build(buffer_type *buffer) {

   sfile sbuffer;

   sbuffer.dest();

   header.build(&sbuffer);

   sbuffer.sput_ushort(fire_site_id);
   sbuffer.sput_ushort(fire_application_id);
   sbuffer.sput_ushort(fire_entity_id);

   sbuffer.sput_ushort(target_site_id);
   sbuffer.sput_ushort(target_application_id);
   sbuffer.sput_ushort(target_entity_id);

   sbuffer.sput_ushort(ammo_site_id);
   sbuffer.sput_ushort(ammo_application_id);
   sbuffer.sput_ushort(ammo_entity_id);

   sbuffer.sput_ushort(event_site_id);
   sbuffer.sput_ushort(event_application_id);
   sbuffer.sput_ushort(event_entity_id);

   sbuffer.sput_int(fire_mission_index);

   sbuffer.sput_double(location[0]);
   sbuffer.sput_double(location[1]);
   sbuffer.sput_double(location[2]);

   burst_data.build(&sbuffer);

   sbuffer.sput_float(velocity[0]);
   sbuffer.sput_float(velocity[1]);
   sbuffer.sput_float(velocity[2]);

   sbuffer.sput_float(range);

   buffer->buffercpy((char *)sbuffer.data, sbuffer.vsize);
}


/* **********************************************************
********************************************************** */
void DISPDU_fire_type::extract(buffer_type *buffer) {

   sfile sbuffer;

   sbuffer.scan_data(buffer);

   header.extract(&sbuffer);

   fire_site_id = sbuffer.scan_ushort();
   fire_application_id = sbuffer.scan_ushort();
   fire_entity_id = sbuffer.scan_ushort();

   target_site_id = sbuffer.scan_ushort();
   target_application_id = sbuffer.scan_ushort();
   target_entity_id = sbuffer.scan_ushort();

   ammo_site_id = sbuffer.scan_ushort();
   ammo_application_id = sbuffer.scan_ushort();
   ammo_entity_id = sbuffer.scan_ushort();

   event_site_id = sbuffer.scan_ushort();
   event_application_id = sbuffer.scan_ushort();
   event_entity_id = sbuffer.scan_ushort();

   fire_mission_index = sbuffer.scan_int();

   location[0] = sbuffer.scan_double();
   location[1] = sbuffer.scan_double();
   location[2] = sbuffer.scan_double();

   burst_data.extract(&sbuffer);

   velocity[0] = sbuffer.scan_float();
   velocity[1] = sbuffer.scan_float();
   velocity[2] = sbuffer.scan_float();

   range = sbuffer.scan_float();
}
